# Load the data
nlsy_math_wide<-read.table("C:/GRE2016/Data/nlsy_math_wide_R.dat", na.string='.')

# Give the variable names
names(nlsy_math_wide)<-c('id','female','lb_wght','anti_k1',
                         'math2','math3','math4','math5','math6','math7','math8',
                         'age2','age3','age4','age5','age6','age7','age8',
                         'men2','men3','men4','men5','men6','men7','men8',
                         'spring2','spring3','spring4','spring5','spring6','spring7','spring8',
                         'anti2','anti3','anti4','anti5','anti6','anti7','anti8')

library(OpenMx)

## No Change Model

no.math.omx <- mxModel("No Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
 latentVars=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7',
                    'dy2','dy3','dy4','dy5','dy6','dy7'),

# Defining true scores
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7'),
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Defining change scores 
mxPath(from=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=60, labels='sigma2_u'),

# Latent variable variance path 
mxPath(from='ly1', 
 arrows=2, free=TRUE, values=60,
 labels='sigma2_1'),

# means 
mxPath(from='one', to='ly1',
 arrows=1, free=TRUE, values=40, labels='gamma_1')

) # close model

no.math.fit <- mxRun(no.math.omx)

summary(no.math.fit)

#free parameters:
#      name matrix   row   col  Estimate Std.Error
#1 sigma2_u      S math2 math2 116.68222  4.548264
#2 sigma2_1      S   ly1   ly1  46.91680  4.834040
#3  gamma_1      M     1   ly1  45.91469  0.323981
#
#observed statistics:  2221 
#estimated parameters:  3 
#degrees of freedom:  2218 
#-2 log likelihood:  17491.9 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      13055.903               17497.90                       NA
#BIC:       2324.321               17512.42                 17502.89



## Constant Change Model

ccm.math.omx <- mxModel("Constant Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
 latentVars=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7',
                    'dy2','dy3','dy4','dy5','dy6','dy7',
                    'g2'),

# Defining true scores
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7'),
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Defining change scores 
mxPath(from=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Constant change component 
mxPath(from='g2', to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=FALSE, values=1),

# Variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=60, labels='sigma2_u'),

# Latent variable covariance matrix path 
mxPath(from=c('ly1','g2'), 
 arrows=2, connect='unique.pairs', free=TRUE, values=c(60, 0, 5),
 labels=c('sigma2_1','sigma_21','sigma2_2')),

# means 
mxPath(from='one', to=c('ly1','g2'),
 arrows=1, free=TRUE, values=c(40, 4), labels=c('gamma_1','gamma_2'))

) # close model

ccm.math.fit <- mxRun(ccm.math.omx)

summary(ccm.math.fit)

#free parameters:
#      name matrix   row   col   Estimate  Std.Error
#1 sigma2_u      S math2 math2 36.2297454 1.90545347
#2 sigma2_1      S   ly1   ly1 64.5618650 5.93162161
#3 sigma_21      S   ly1    g2 -0.1814791 1.25838634
#4 sigma2_2      S    g2    g2  0.7325619 0.34530595
#5  gamma_1      M     1   ly1 35.2673601 0.35542020
#6  gamma_2      M     1    g2  4.3393281 0.08835106
#
#observed statistics:  2221 
#estimated parameters:  6 
#degrees of freedom:  2215 
#-2 log likelihood:  15937.39 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:     11507.3857               15949.39                       NA
#BIC:       790.3181               15978.42                 15959.36



## Proportional Change Model

pcm.math.omx <- mxModel("Proportional Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
 latentVars=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7',
                    'dy2','dy3','dy4','dy5','dy6','dy7'),

# Defining true scores
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7'),
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Defining change scores 
mxPath(from=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=60, labels='sigma2_u'),

# Latent variable variances
mxPath(from=c('ly1'), 
 arrows=2, free=TRUE, values=60,
 labels='sigma2_1'),

# Proportional Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=TRUE, values=0,
 labels='pi'),

# means 
mxPath(from='one', to='ly1',
 arrows=1, free=TRUE, values=40, labels='gamma_1')

) # close model

pcm.math.fit <- mxRun(pcm.math.omx)

summary(pcm.math.fit)

#free parameters:
#      name matrix   row   col    Estimate   Std.Error
#1       pi      A   dy2   ly1  0.09184619 0.002005413
#2 sigma2_u      S math2 math2 42.99436557 1.699917644
#3 sigma2_1      S   ly1   ly1 41.64809640 2.598682411
#4  gamma_1      M     1   ly1 36.58712549 0.307059559
#
#observed statistics:  2221 
#estimated parameters:  4 
#degrees of freedom:  2217 
#-2 log likelihood:  16072.17 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      11638.172               16080.17                       NA
#BIC:        911.428               16099.53                 16086.82


## Dual Change Model

dcm.math.omx <- mxModel("Dual Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
 latentVars=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7',
                    'dy2','dy3','dy4','dy5','dy6','dy7',
                    'g2'),

# Defining true scores
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7'),
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Defining change scores 
mxPath(from=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Constant change component 
mxPath(from='g2', to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=FALSE, values=1),

# Variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=60, labels='sigma2_u'),

# Latent variable covariance matrix path 
mxPath(from=c('ly1','g2'), 
 arrows=2, connect='unique.pairs', free=TRUE, values=c(60, 0, 5),
 labels=c('sigma2_1','sigma_21','sigma2_2')),

# Proportional Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=TRUE, values=0,
 labels='pi'),

# means 
mxPath(from='one', to=c('ly1','g2'),
 arrows=1, free=TRUE, values=c(40, 4), labels=c('gamma_1','gamma_2'))

) # close model

dcm.math.fit <- mxRun(dcm.math.omx)

summary(dcm.math.fit)

#free parameters:
#      name matrix   row   col   Estimate  Std.Error
#1       pi      A   dy2   ly1 -0.2410135 0.01798447
#2 sigma2_u      S math2 math2 30.8184687 1.69889645
#3 sigma2_1      S   ly1   ly1 71.9004194 6.54300591
#4 sigma_21      S   ly1    g2 13.7461381 1.69771623
#5 sigma2_2      S    g2    g2  5.6019870 0.83539419
#6  gamma_1      M     1   ly1 32.5327261 0.43382728
#7  gamma_2      M     1    g2 15.2221890 0.81376519
#
#observed statistics:  2221 
#estimated parameters:  7 
#degrees of freedom:  2214 
#-2 log likelihood:  15791.21 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:     11363.2093               15805.21                       NA
#BIC:       650.9801               15839.08                 15816.85


## Dual Change Model with Dynamic Noise

dcmn.math.omx <- mxModel("Dual Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
 latentVars=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7',
                    'dy2','dy3','dy4','dy5','dy6','dy7',
                    'g2'),

# Defining true scores
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7'),
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Defining change scores 
mxPath(from=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Constant change component 
mxPath(from='g2', to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=FALSE, values=1),

# Variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=60, labels='sigma2_u'),

# Latent variable covariance matrix path 
mxPath(from=c('ly1','g2'), 
 arrows=2, connect='unique.pairs', free=TRUE, values=c(60, 0, 5),
 labels=c('sigma2_1','sigma_21','sigma2_2')),

# Proportional Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=TRUE, values=0,
 labels='pi'),

# means 
mxPath(from='one', to=c('ly1','g2'),
 arrows=1, free=TRUE, values=c(40, 4), labels=c('gamma_1','gamma_2')),

# Dynamic Noise paths
mxPath(from=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=2, free=TRUE, values=1, labels='sigma2_dy')

) # close model

dcmn.math.fit <- mxRun(dcmn.math.omx)

summary(dcmn.math.fit)

#free parameters:
#       name matrix   row   col    Estimate  Std.Error
#1        pi      A   dy2   ly1  -0.2341707 0.01771669
#2  sigma2_u      S math2 math2  39.0940001 3.96858857
#3  sigma2_1      S   ly1   ly1  68.8004549 6.81384806
#4 sigma2_dy      S   dy2   dy2 -10.2028461 4.15651557
#5  sigma_21      S   ly1    g2  14.0569917 1.66738827
#6  sigma2_2      S    g2    g2   6.9160577 0.95446647
#7   gamma_1      M     1   ly1  32.6313207 0.44451725
#8   gamma_2      M     1    g2  14.9067336 0.80429675
#
#observed statistics:  2221 
#estimated parameters:  8 
#degrees of freedom:  2213 
#-2 log likelihood:  15785.79 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:     11359.7860               15801.79                       NA
#BIC:       652.3953               15840.49                 15815.09


## Linear Change Model
lcm.math.omx <- mxModel("Linear Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
 latentVars=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7',
                    'dy2','dy3','dy4','dy5','dy6','dy7',
                    'g2', 'g3'),

# Defining true scores
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6','ly7'),
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects 
mxPath(from=c('ly1','ly2','ly3','ly4','ly5','ly6'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Defining change scores 
mxPath(from=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 to=c('ly2','ly3','ly4','ly5','ly6','ly7'),
 arrows=1, free=FALSE, values=1),

# Constant change component 
mxPath(from='g2', to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=FALSE, values=1),

# Linear change factor 
mxPath(from='g3', to=c('dy2','dy3','dy4','dy5','dy6','dy7'),
 arrows=1, free=FALSE, values=c(2,4,6,8,10,12)),

# Variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=60, labels='sigma2_u'),

# Latent variable covariance matrix path 
mxPath(from=c('ly1','g2','g3'), 
 arrows=2, connect='unique.pairs', free=TRUE, values = c(60, 0, 0, 5, 0, 1),
 labels=c('sigma2_1','sigma_21','sigma_31','sigma2_2','sigma_32', 'sigma2_3')),

# means 
mxPath(from='one', to=c('ly1','g2','g3'),
 arrows=1, free=TRUE, values=c(40, 4, -.5), labels=c('gamma_1','gamma_2','gamma_3'))

) # close model

lcm.math.fit <- mxRun(lcm.math.omx)

summary(lcm.math.fit)

#free parameters:
#       name matrix   row   col    Estimate  Std.Error
#1  sigma2_u      S math2 math2 32.92546512 1.84743701
#2  sigma2_1      S   ly1   ly1 73.59577590 6.87506029
#3  sigma_21      S   ly1    g2 -5.21857128 3.34086262
#4  sigma2_2      S    g2    g2 -1.54687744 2.40962000
#5  sigma_31      S   ly1    g3  0.66811915 0.48009871
#6  sigma_32      S    g2    g3  0.45538126 0.34470745
#7  sigma2_3      S    g3    g3 -0.08180166 0.05163223
#8   gamma_1      M     1   ly1 32.96639533 0.41276729
#9   gamma_2      M     1    g2  7.73617413 0.29463329
#10  gamma_3      M     1    g3 -0.53278701 0.04367827
#
#observed statistics:  2221 
#estimated parameters:  10 
#degrees of freedom:  2211 
#-2 log likelihood:  15793.56 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:     11371.5554               15813.56                       NA
#BIC:       673.8415               15861.94                 15830.18






